#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "ptwrapper_o.h"
#include "debug.h"

char *progname = "cat";

/*D

  ptcat - SUT cat

Description:

ptcat concatenates files from several hosts to the standard output.

Command Line Arguments:

ptcat [-m <machine_file>|-M <machine_list>] [-nbsuveth] [<file1> <file2> <file3>...]

.N machine_args
. -nbsuvet - same as for cat(1)
. -h - print the hostname as a header for the output from each host
- [<file1> <file2> <file3>...] - the list of files to concatenate

Environment Variables:

.N machine_envs

Examples:

To concatenate the contents of the file "/etc/passwd" to the standard output
on the machines specified in the file "destination_machines" use

.vb
ptcat -m destination_machines /etc/passwd
.ve

To do the same concatenation, specifying the machines on the command line, use

.vb
ptcat -M "host1 host2 host3" /etc/passwd
.ve

To concatenate the contents of the file "/etc/passwd" to the standard output
with the hostname preceeding each line printed, use

.vb
ptcat -m destination_machines -h /etc/passwd
.ve

.seealso cat(1)
  
 D*/
int check_args(int argc, char *argv[], char *cmd)
{
	int len;
    int i, c;
    char opt[4];

	snprintf(cmd, MAX_CMD_LEN, "%s ", progname);
	len = strlen(cmd);

    while ((c = getopt(argc, argv, "nbsuveth")) != -1){
        switch (c){
            case 'n':
            case 'b':
            case 's':
            case 'u':
            case 'v':
            case 'e':
            case 't':
                len += 3;
                if (len >= MAX_CMD_LEN)
                    return -2;
                sprintf(opt, "-%c ", c);
                strcat(cmd, opt);
                break;
            case 'h':
                add_hostname = 1;
                break;
            default:
                return -1;
        }
    }

    for (i = optind; i < argc; i++){
        len += strlen(argv[i]) + 1;
        if (len >= MAX_CMD_LEN)
            return -2;
		strcat(cmd, argv[i]);
        strcat(cmd, " ");
    }
    
    pt_debug("the command is '%s'\n", cmd);

    return 0;
}
