/* Argument checker for chown */
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

char *progname = "chown";

/*D

  ptchown - SUT chown

Description:

ptchown changes the owner and group of a list of files on specified hosts.

Command Line Arguments:

ptchown [-m <machine_file>|-M <machine_list>] [-Rfh] <user>[:<group>] <file1> [<file2> <file3>...]

.N machine_args

. -R - change ownership recursively
. -f - suppress error messages
. -h - change the ownership of symbolic links instead of the files to which they point
. user - the username of the new owner of the files
. group - the groupname of the new group of the files (optional)
- <file1> [<file2> <file3>...] - the list of files to reown

Environment Variables:

.N machine_envs

Examples:

To change the owner of a file "myfile" to johndoe on the machines
specified in the file "destination_machines" use

.vb
ptchown -m destination_machines johndoe myfile
.ve

To make the same change, specifying the machines on the command line, use

.vb
ptchown -M "host1 host2 host3" johndoe myfile
.ve

To change the owner and group of all files and directories starting with 'r' 
in directory "/home/johndoe/project" to root and root, use

.vb
ptchown -m destination_machines -R root:root "/home/johndoe/project/r*"
.ve

.seealso chown(1)
  
 D*/
int check_args(int argc, char *argv[])
{
    int c;

    while ((c = getopt(argc, argv, "Rfh")) != -1) {
        switch (c) {
            case 'R':
            case 'f':
            case 'h':
                continue;
            default:
                return -1;
        }
    }

    if (argc - optind < 2)
        return -1;
    
    return 0;
}
