#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "nodes.h"

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif /* HAVE_CONFIG_H */

#include "debug.h"

static int get_ring_size()
{
	int size;
	FILE *f;

	f = popen("mpdringsize -e", "r");
	if (f == NULL)
		return -1;

	fscanf(f, "mpdringsize=%d", &size);

	pclose(f);

	return size; 
}

#define BUF_SIZE    1024

int nodes_init(int argc, char *argv[], int min_size,
		       char **filename, int *size, int *all, int *optind)
{
    int fd;
    FILE *f;
	char tmpmf[L_tmpnam];
    char buf[BUF_SIZE];
	char *machine_file = NULL;
	struct nodelist *nl = NULL;

	*all = 0;
	*size = min_size;
	*filename = NULL;
	
	/* Preliminary environment/argument check */
	machine_file = getenv("MACHINE_FILE");

	/* Check the command line */
    if ((argc > 1) && (strcmp(argv[1], "-all") == 0)) {
        *all = 1;
        *optind = 2;
    } else if ((argc > 1) && (strcmp(argv[1], "-m") == 0)){
		/* We were given a machine file filename */
		nl = nodelist_parse_file(argv[2]);
		*optind = 3;
	} else if ((argc > 1) && (strcmp(argv[1], "-M") == 0)){
		/* We were given the machines on the command line */
		nl = nodelist_parse(argv[2]);
		*optind = 3;
	} else if (machine_file != NULL) {
		/* We were given a machine file filename */
		nl = nodelist_parse_file(machine_file);
		*optind = 1;
	} else {
		fprintf(stderr, "No machines or machine file were given or "
					    "MACHINE_FILE is not set\n");
		return 1;
	}

	if (!(*all)) {
		if (nl == NULL) {
			fprintf(stderr, "Unable to parse machine names\n");
			return -1;
		}

		/* Open a temporary file to send to mpirun */
		if (tmpnam(tmpmf) == NULL){
			fprintf(stderr, "Unable to generate temporary file\n");
			return -1;
		}

		*filename = strdup(tmpmf);
		if (*filename == NULL) {
			fprintf(stderr, "Unable to allocate space for temporary filename\n");
			return -1;
		}

        /* Create the temp file somewhat securely */
        fd = open(tmpmf, O_WRONLY|O_CREAT, 0600);
        if (fd < 0) {
			fprintf(stderr, "Unable to generate temporary file\n");
			return -1;
        }

		f = fdopen(fd, "w");
		if (f == NULL){
			fprintf(stderr, "Unable to generate temporary file\n");
			return -1;
		}

	    /* Write out the intermediate machine file */
        pt_debug("Exec'ing on these machines:\n", buf);
        nodelist_reset(nl);
        while (nodelist_next(nl, buf, BUF_SIZE) != NULL){
            fprintf(f, "%s\n", buf);
            pt_debug("%s\n", buf);
            (*size)++;
        }

        fclose(f);
    } else {
        *size = get_ring_size();
		if (*size < 0) {
			fprintf(stderr, "Cannot get ring size\n");
			return -1;
		}
    }

	pt_debug("num_machines = %d\n", *size);

	return 0;
}
