#include <time.h>
#include <stdlib.h>
#include <stdarg.h>
#include "ptfps.h"
#include "debug.h"

expression_t *expression_term(char * type, char *args[])
{
    option_t tmp;
    option_t *opt;
    expression_t *e;

    tmp.name = type;

    opt = bsearch(&tmp, options, num_options, sizeof(option_t), optcmp);
    if (opt == NULL)
        return NULL;

	e = malloc(sizeof(expression_t));
	if (e == NULL)
		return NULL;

    if (opt->parse(args, &(e->term_value)) < 0) {
        free(e);
        return NULL;
    }

    e->printed_header = 0;
	e->conj = CONJ_NONE;
	e->negated = 0;
    e->type = type;
	e->left = NULL;
	e->right = NULL;

	return e;
}

expression_t *expression_conj(enum conj conj, expression_t *l, expression_t *r)
{
    expression_t *e;

    switch (conj) {
        case CONJ_NONE:
            return NULL;
        case CONJ_AND:
        case CONJ_OR:
            e = malloc(sizeof(expression_t));
            if (e == NULL)
                return NULL;

            e->printed_header = 0;
            e->conj = conj;
            e->negated = 0;
            e->type = NULL;
            e->left = l;
            e->right = r;

            return e;
    }

    return NULL;
}

void expression_destroy(expression_t *e)
{
	if (e == NULL)
		return;

	if (e->conj != CONJ_NONE) {
        expression_destroy(e->left);
        expression_destroy(e->right);
    }
    
    free(e);
}
	
expression_t *expression_negate(expression_t *e)
{
	e->negated = e->negated ? 0 : 1;
	return e;
}

/* If all else fails, return this */
#define DEFAULT		1
int expression_evaluate(expression_t *e, process_t *p, char *buf, size_t size)
{
    int val;
    option_t tmp;
    option_t *opt;

	if (e == NULL)
		return DEFAULT;

    pt_debug("Evaluating an expression\n");
    switch (e->conj) {
        case CONJ_AND:
            return (expression_evaluate(e->left, p, buf, size) &&
                    expression_evaluate(e->right, p, buf, size));
        case CONJ_OR:
            return (expression_evaluate(e->left, p, buf, size) || 
                    expression_evaluate(e->right, p, buf, size));
        case CONJ_NONE:
            tmp.name = e->type;
            opt = bsearch(&tmp, options, num_options, sizeof(option_t), optcmp);
            if (opt == NULL)
                return DEFAULT;

            val = opt->eval(e, p, buf, size);
            if (e->negated)
                val = !val;

            return val;
	}

    return DEFAULT;
}
