#include <stdio.h>
#include <time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <signal.h>
#include <string.h>
#include <sys/wait.h>
#include <time.h>
#include <pwd.h>
#include <grp.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/fault.h>
#include <sys/syscall.h>
#include <sys/procfs.h>
#include <sys/sysmp.h>
#include "get_date.h"
#include "ptfps.h"
#include "ptfps_irix.h"
#include "debug.h"

#define MAX_LINE_LEN	4096
#define MAX_TIME_LEN	16

static int got_meminfo = 0;
static __uint32_t physpages = 0;
				
process_t *get_next_process()
{
	static DIR *d = NULL;
	static process_t pr;
    /* sizeof("/proc/pinfo/") + num digits in proc files + 1 (for the null) */
    static char filename[12 + 10 + 1]; 
    struct dirent *de;
	
	if (d == NULL){
        d = opendir("/proc/pinfo");
		
		if (d == NULL){
			pt_debug("Unable to open /proc/pinfo\n");
			return NULL;
		}

        if (!got_meminfo) {
            struct rminfo rmi;

            if (sysmp(MP_KERNADDR, MPSA_RMINFO, &rmi) >= 0) {
                got_meminfo = 1;
                physpages = rmi.physmem / getpagesize();
            }
        }

	}

    while ((de = readdir(d)) != NULL) {
        int fd, dummy;

        /* Ignore . and .. */
        if (sscanf(de->d_name, "%d", &dummy) == 1) {
            snprintf(filename, 23, "/proc/pinfo/%s", de->d_name);
            fd = open(filename, O_RDONLY);
            if (fd < 0) {
                pt_debug("Unable to open %s\n", filename);
                continue;
            }
            if (ioctl(fd, PIOCPSINFO, &pr.psinfo) < 0) {
                pt_debug("ioctl on %s failed\n", filename);
                continue;
            }
            pr.have_cred = (ioctl(fd, PIOCCRED, &pr.cred) >= 0);
            close(fd);
            break;
        }
    }

	return de == NULL ? NULL : &pr;
}

/* UID's */
uid_t get_ruid(process_t *p)
{
	return p->have_cred ? p->cred.pr_ruid : p->psinfo.pr_uid;
}
uid_t get_euid(process_t *p)
{
	return p->have_cred ? p->cred.pr_euid : p->psinfo.pr_uid;
}
uid_t get_suid(process_t *p)
{
	return p->have_cred ? p->cred.pr_suid : p->psinfo.pr_uid;
}

/* GID's */
gid_t get_rgid(process_t *p)
{
	return p->have_cred ? p->cred.pr_rgid : p->psinfo.pr_gid;
}

gid_t get_egid(process_t *p)
{
	return p->have_cred ? p->cred.pr_egid : p->psinfo.pr_gid;
}

gid_t get_sgid(process_t *p)
{
	return p->have_cred ? p->cred.pr_sgid : p->psinfo.pr_gid;
}

/* Username's */
char *get_rusername(process_t *p)
{
    struct passwd *pwd;

    pwd = getpwuid(get_ruid(p));
    if (pwd == NULL)
        return "N/A";

    return pwd->pw_name;
}

char *get_eusername(process_t *p)
{
    struct passwd *pwd;

    pwd = getpwuid(get_euid(p));
    if (pwd == NULL)
        return "N/A";

    return pwd->pw_name;
}

char *get_susername(process_t *p)
{
    struct passwd *pwd;

    pwd = getpwuid(get_suid(p));
    if (pwd == NULL)
        return "N/A";

    return pwd->pw_name;
}

/* Group names */
char *get_rgroupname(process_t *p)
{
    struct group *gr;

    gr = getgrgid(get_rgid(p));
    if (gr == NULL)
        return "N/A";

    return gr->gr_name;
}

char *get_egroupname(process_t *p)
{
    struct group *gr;

    gr = getgrgid(get_egid(p));
    if (gr == NULL)
        return "N/A";

    return gr->gr_name;
}

char *get_sgroupname(process_t *p)
{
    struct group *gr;

    gr = getgrgid(get_sgid(p));
    if (gr == NULL)
        return "N/A";

    return gr->gr_name;
}

/* Etc */
char *get_name(process_t *p)
{
	return p->psinfo.pr_psargs;
}

static char *brute_tty(dev_t dev)
{
    DIR *d;
    struct dirent *de;
    struct stat st;
    static char buf[50];

    d = opendir("/dev/pts");
    if (d == NULL)
        return "?";

    while ((de = readdir(d)) != NULL) {
        buf[0] = '\0';
        strcat(buf, "/dev/pts/");
        strncat(buf, de->d_name, 50-sizeof("/dev/pts/"));
        if (stat(buf, &st) == 0) {
            if (st.st_rdev == dev) {
                strcpy(buf, "pts/");
                strncat(buf, de->d_name, 50-sizeof("pts/"));
                return buf;
            }
        }
    }

    d = opendir("/dev");
    if (d == NULL)
        return "?";

    while ((de = readdir(d)) != NULL) {
        strcpy(buf, "/dev/");
        strncat(buf, de->d_name, 50-sizeof("/dev/"));
        if (stat(buf, &st) == 0) {
            if (st.st_rdev == dev) {
                strncpy(buf, de->d_name, 50);
                return buf;
            }
        }
    }

    return "?";
}

char *get_tty(process_t *p)
{
    if (p->psinfo.pr_ttydev == PRNODEV)
        return "?";

    return brute_tty(p->psinfo.pr_ttydev);
}

pid_t get_pid(process_t *p)
{
	return p->psinfo.pr_pid;
}
	
pid_t get_ppid(process_t *p)
{
	return p->psinfo.pr_ppid;
}
	
time_t get_stime(process_t *p)
{
	return p->psinfo.pr_start.tv_sec;
}

unsigned long get_time(process_t *p)
{
	return (unsigned long)p->psinfo.pr_time.tv_sec;
}

double get_pcpu(process_t *p)
{
    time_t now;

    time(&now);
 	return (double)p->psinfo.pr_time.tv_sec / 
           (double)(now - p->psinfo.pr_start.tv_sec);
}

double get_pmem(process_t *p)
{
	return (double)p->psinfo.pr_rssize / (double)physpages;
}

unsigned long get_rss(process_t *p)
{
    return (unsigned long)p->psinfo.pr_rssize;
}

unsigned long get_vsz(process_t *p)
{
    return (unsigned long)p->psinfo.pr_size;
}

unsigned long get_flags(process_t *p)
{
    return p->psinfo.pr_flag;
}

int get_pri(process_t *p)
{
    return (int)(p->psinfo.pr_pri);
}

int get_nice(process_t *p)
{
    return (int)(p->psinfo.pr_nice);
}

char *get_wchan(process_t *p)
{
    return "";
}

status_t get_state(process_t *p)
{
    switch (p->psinfo.pr_state) {
        case 'R':
        case '0':
            return ST_RUNNABLE;
        case 'S':
            return ST_SLEEPING;
        case 'T':
            return ST_STOPPED;
        case 'Z':
            return ST_ZOMBIE;
        default:
            return ST_UNKNOWN;
    }
}

char *get_cmd(process_t *p)
{
    return p->psinfo.pr_fname;
}
