#ifndef PTFPS_PROCESS_H
#define PTFPS_PROCESS_H

#ifdef HAVE_CONFIG_H
#   include "config.h"
#endif /* HAVE_CONFIG_H */

#ifdef HAVE_PROCPS
#include "ptfps_procps.h"
#elif HAVE_IRIX
#include "ptfps_irix.h"
#elif HAVE_LINUX
#include "ptfps_linux.h"
#elif HAVE_SOLARIS
#include "ptfps_solaris.h"
#elif HAVE_AIX
#include "ptfps_aix.h"
#else
#error "Unsupported operating system"
#endif /* HAVE_PROCPS */

/* Status */
typedef enum {
    ST_RUNNABLE,
    ST_SLEEPING,
    ST_STOPPED,
    ST_ZOMBIE,
    ST_UNKNOWN
} status_t;

extern process_t *get_next_process();

/* The idea here is to have an opaque object, a process_t, that represents
 * a process from which you can grab the following information. */
extern uid_t         get_ruid(process_t *p);
extern uid_t         get_euid(process_t *p);
extern uid_t         get_suid(process_t *p);

extern gid_t         get_rgid(process_t *p);
extern gid_t         get_egid(process_t *p);
extern gid_t         get_sgid(process_t *p);

extern char *        get_rusername(process_t *p);
extern char *        get_eusername(process_t *p);
extern char *        get_susername(process_t *p);

extern char *        get_rgroupname(process_t *p);
extern char *        get_egroupname(process_t *p);
extern char *        get_sgroupname(process_t *p);

extern char *        get_name(process_t *p);
extern char *        get_cmd(process_t *p);
extern char *        get_tty(process_t *p);
extern status_t      get_state(process_t *p);
extern char *        get_wchan(process_t *p);
extern pid_t         get_pid(process_t *p);
extern pid_t         get_ppid(process_t *p);
extern time_t        get_stime(process_t *p);
extern double        get_pcpu(process_t *p);
extern double        get_pmem(process_t *p);
extern unsigned long get_time(process_t *p);
extern unsigned long get_rss(process_t *p);
extern unsigned long get_vsz(process_t *p);
extern unsigned long get_flags(process_t *p);
extern int           get_pri(process_t *p);
extern int           get_nice(process_t *p);

#endif /* PTFPS_PROCESS_H */
