#include <sys/types.h>
#include <signal.h>
#include <stdlib.h>
#include "signame.h"

static struct signame {
	char *name;
	int value;
} signals[] = {
	{"ABRT" , SIGABRT},
	{"ALRM" , SIGALRM},
	{"BUS" , SIGBUS},
	{"CHLD" , SIGCHLD},
	{"CLD" , SIGCLD},
	{"CONT" , SIGCONT},
	{"FPE" , SIGFPE},
	{"HUP" , SIGHUP},
	{"ILL" , SIGILL},
	{"INT" , SIGINT},
	{"IO" , SIGIO},
	{"IOT" , SIGIOT},
	{"KILL" , SIGKILL},
	{"PIPE" , SIGPIPE},
	{"POLL" , SIGPOLL},
	{"PROF" , SIGPROF},
	{"QUIT" , SIGQUIT},
	{"SEGV" , SIGSEGV},
	{"SIGABRT" , SIGABRT},
	{"SIGALRM" , SIGALRM},
	{"SIGBUS" , SIGBUS},
	{"SIGCHLD" , SIGCHLD},
	{"SIGCLD" , SIGCLD},
	{"SIGCONT" , SIGCONT},
	{"SIGFPE" , SIGFPE},
	{"SIGHUP" , SIGHUP},
	{"SIGILL" , SIGILL},
	{"SIGINT" , SIGINT},
	{"SIGIO" , SIGIO},
	{"SIGIOT" , SIGIOT},
	{"SIGKILL" , SIGKILL},
	{"SIGPIPE" , SIGPIPE},
	{"SIGPOLL" , SIGPOLL},
	{"SIGPROF" , SIGPROF},
	{"SIGQUIT" , SIGQUIT},
	{"SIGSEGV" , SIGSEGV},
	{"SIGSTOP" , SIGSTOP},
	{"SIGTERM" , SIGTERM},
	{"SIGTRAP" , SIGTRAP},
	{"SIGTSTP" , SIGTSTP},
	{"SIGTTIN" , SIGTTIN},
	{"SIGTTOU" , SIGTTOU},
	{"SIGURG" , SIGURG},
	{"SIGUSR1" , SIGUSR1},
	{"SIGUSR2" , SIGUSR2},
	{"SIGVTALRM" , SIGVTALRM},
	{"SIGWINCH" , SIGWINCH},
	{"SIGXCPU" , SIGXCPU},
	{"SIGXFSZ" , SIGXFSZ},
	{"STOP" , SIGSTOP},
	{"TERM" , SIGTERM},
	{"TRAP" , SIGTRAP},
	{"TSTP" , SIGTSTP},
	{"TTIN" , SIGTTIN},
	{"TTOU" , SIGTTOU},
	{"URG" , SIGURG},
	{"USR1" , SIGUSR1},
	{"USR2" , SIGUSR2},
	{"VTALRM" , SIGVTALRM},
	{"WINCH" , SIGWINCH},
	{"XCPU" , SIGXCPU},
	{"XFSZ" , SIGXFSZ},
};

static int sigcmp(const void *a, const void *b)
{
	const struct signame *x = a, *y = b;
	return strcmp(x->name, y->name);
}

int signame_to_signum(char *name)
{
	struct signame tmp;
	struct signame *x;

    tmp.name = name;

	x = bsearch(&tmp, signals, sizeof(signals)/sizeof(signals[0]), 
			    sizeof(struct signame), sigcmp);

	if (x == NULL)
		return -1;

	return x->value;
}
